<?php

class image{
	/**
	 * 获得缩略图路径
	 * @param string $old 原始路径
	 */
	public function getThumbPath($old,$attrib){
		$width=kc_val($attrib,'width');
		$height=kc_val($attrib,'height');
		$fileName=basename($old);
		$path=dirname($old).'/thumb/'.$width.'x'.$height.'/'.$fileName;
		return $path;
	}
	/**
	 * 图像处理
	 * @param string $s       图片路径
	 * @param array  $attrib  图片属性 需要有width和height属性 外加一些滤镜
	 * @param bool $is 是否跳过生成判断，默认如果有图片，则跳过生成缩略图
	 * @return string 返回缩略图地址
	*/
	public function thumb($s,$attrib,$is=false){
		//画布大小
		$width=kc_val($attrib,'width');
		$height=kc_val($attrib,'height');
		//若文件不存在 直接返回值
		if(!@is_file(ROOT.$s)) return $s;		
		//原图大小
		list($_width, $_height)=@getimagesize(ROOT.$s);
		//如果获取原图宽高失败，则退出截图程序
		if(empty($_width) || empty($_height)){
			return $s;
		}		
		//无 输出宽度 属性的时候，补充宽度值
		$width=kc_validate($width,2) ? $width : round($height*$_width/$_height);
		//无 输出高度 属性的时候，补充宽度值
		$height=kc_validate($height,2) ? $height : round($width*$_height/$_width);
		//如果原图大小小于缩略图要求，则直接返回其值,即不做缩略图操作
		if ($width>=$_width && $height>=$_height) {
			return $s;
		}
		// 根据宽高获得新的图片路径地址
		$newimg=$this->getThumbPath($s,array('width'=>$width,'height'=>$height));//新文件
		// 新文件的目录
		$newdir=dirname($newimg);
		//如果缩略图文件存在，并$is判断为false时，则直接返回缩略图文件地址
		if(is_file(ROOT.$newimg) && !$is){
			return PATH.$newimg;
		}
		//指定背景颜色 则对图片缩图处理
		$background=kc_validate(kc_val($attrib,'background'),13) ? $attrib['background'] :'';
		if (!empty($background)) {//缩图
			$pose=kc_val($attrib,'post');
			if ($_width/$_height >= $width/$height) {
				$w=$width;
				$h=round(($_height/$_width)*$width);
				$x=0;
				$y=round(($height-$h)/2);
				switch($pose){
					case 'top':$y=0;break;
					case 'bottom':$y=$height-$w;break;
				}
			}else{
				$w=round(($_width/$_height)*$height);
				$h=$height;
				$x=round(($width-$w)/2);
				$y=0;
				switch($pose){
					case 'left':$x=0;break;
					case 'right':$x=$width-$w;break;
				}
			}
		}else{//裁图
			if($_width/$_height>=$width/$height){
				// 裁图宽高
				$w=round(($_width/$_height)*$height);
				$h=$height;
				// 插图位置
				$x=round(($width-$w)/2);
				$y=0;
			}else{
				$w=$width;
				$h=round(($_height/$_width)*$width);
				$x=0;
				$y=round(($height-$h)/2);
			}
			//裁图位置
			$pose=kc_val($attrib,'pose');
			switch ($pose){
				case 'top':$y=0;break;
				case 'left':$x=0;break;
				case 'right':$x=$x*2;break;
				case 'bottom':$y=$y*2;break;
				case 'lefttop':$x=0;$y=0;break;
				case 'topleft':$x=0;$y=0;break;
				case 'righttop':$x=$x*2;$y=0;break;
				case 'topright':$x=$x*2;$y=0;break;
				case 'leftbottom':$x=0;$y=$y*2;break;
				case 'bottomleft':$x=0;$y=$y*2;break;
				case 'rightbottom':$x=$x*2;$y=$y*2;break;
				case 'bottomright':$x=$x*2;$y=$y*2;break;
				default:
					if (kc_validate($pose,'/^\-?\d+\,\-?\d+$/')){
						list($x,$y)=explode(',',$pose);
					}
			}
		}

		$size=@getimagesize(ROOT.$s);
		$ext=strtolower(substr(strrchr(kc_val($size,'mime'), '/'),1));

		if(in_array($ext,array('jpg','jpeg','gif','png'))){
			$fext= ($ext=='jpg') ? 'jpeg' : strtolower($ext);
		}else{
			return $s;
		}

		$func='imagecreatefrom'.$fext;
		$source=$func(ROOT.$s);
		//创建空白图
		$im=imagecreatetruecolor($width, $height);
		//填充颜色
		if (!empty($background)) {
			$color=$this->hex2rgb($im,$background);
			imagefill($im,0,0,$color);
		}
		//拷贝图像新建
		imagecopyresampled($im,$source,$x,$y,0,0,$w,$h,$_width,$_height);
		//重设长宽，减1是位置像素问题
		$w1=$width-1;
		$h1=$height-1;
		//置空
		$padding=kc_val($attrib,'padding');
		if(isset($padding{0})){
			$c1=kc_validate(kc_val($attrib,'padding-color'),13)?$attrib['padding-color']:'FFFFFF';//颜色
			$s1=kc_validate(kc_val($attrib,'padding-size'),2)?$attrib['padding-size']:0;//边框的厚度
			$r1=kc_validate(kc_val($attrib,'padding-r'),2)?$attrib['padding-r']:5;//圆角(半径)
			$padding_filter=kc_val($attrib,'padding-filter');//滤镜
			$padding_filter_color=kc_validate(kc_val($attrib,'padding-filter-color'),13) ? $attrib['padding-filter-color'] : '#666666';//颜色
			$color=$this->hex2rgb($im,$c1);//转换颜色

			for($i=0;$i<$s1;$i++){
				imagerectangle($im,$i,$i,$w1-$i,$h1-$i,$color);//方框填充
			}

			switch(strtolower($padding)){
				case 'rectangle'://矩形
					//滤镜
					switch(strtolower($padding_filter)){
						case 'shadow':
							for($i=0;$i<$s1;$i++){
								$color=$this->hex2rgb($im,$padding_filter_color,(1-$i/$s1)*100);
								imageline($im,$s1+$i,$h1-$s1+$i,$w1-$s1+$i,$h1-$s1+$i,$color);
								imageline($im,$w1-$s1+$i,$s1+$i,$w1-$s1+$i,$h1-$s1+$i,$color);
							}
						break;
					}

				break;

				case 'fillet'://圆角矩形
					$d1=$s1+$r1-1;//位置
					for($i=0;$i<$r1+$s1;$i++){
						//画ARC
						$r=($r1+$i)*2-1;
						imagearc($im, $d1,    $d1,    $r,$r, 180,270,$color);//左上
						imagearc($im, $w1-$d1,$d1,    $r,$r, 270,360,$color);//右上
						imagearc($im, $w1-$d1,$h1-$d1,$r,$r,   0, 90,$color);//右下
						imagearc($im, $d1,    $h1-$d1,$r,$r,  90,180,$color);//左下
					}
					//滤镜
					switch(strtolower($padding_filter)){
						case 'shadow':
							for($i=0;$i<$s1;$i++){
								$r=($r1+$i)*2-1;
								$color=$this->hex2rgb($im,$padding_filter_color,(1-$i/$s1)*100);
								imageline($im,$d1+$i,$h1-$s1+$i,$w1-$d1,$h1-$s1+$i,$color);
								imageline($im,$w1-$s1+$i,$d1+$i+1,$w1-$s1+$i,$h1-$d1,$color);
								imagearc($im, $w1-$d1,$h1-$d1,$r,$r,   0, 90,$color);//右下

							}
						break;
					}
				break;
				case 'chamfer'://斜切矩形
					$d1=$s1+$r1-1;//位置
					for($i=0;$i<$r1;$i++){
						imageline($im,$w1-$d1+$i,$s1,$w1-$s1,$d1-$i,$color);//右上
						imageline($im,$w1-$s1,$h1-$d1+$i,$w1-$d1+$i,$h1-$s1,$color);//右下
						imageline($im,$d1-$i,    $h1-$s1,$s1,    $h1-$d1+$i,$color);//左下
						imageline($im,$s1,    $d1-$i,$d1-$i,    $s1,$color);//左上
					}
					//滤镜
					switch(strtolower($padding_filter)){
						case 'shadow':
							for($i=0;$i<$s1;$i++){
								$r=($r1+$i)*2-1;
								$color=$this->hex2rgb($im,$padding_filter_color,(1-$i/$s1)*100);
								imageline($im,$d1+$i+1,$h1-$s1+$i,$w1-$d1,$h1-$s1+$i,$color);
								imageline($im,$w1-$s1+$i,$d1+$i+1,$w1-$s1+$i,$h1-$d1,$color);
								imageline($im,$w1-$s1+$i,$h1-$d1,$w1-$d1,$h1-$s1+$i,$color);//右下
							}
						break;
					}
				break;
			}
		}

		$border=kc_val($attrib,'border');
		if(isset($border{0})){

			$c1=kc_validate(kc_val($attrib,'border-color'),13)?$attrib['border-color']:'000000';//颜色
			$m1=kc_validate(kc_val($attrib,'border-margin'),2)?$attrib['border-margin']:0;//移位
			$r1=kc_validate(kc_val($attrib,'border-r'),2)?$attrib['border-r']:5;//圆角(半径)
			$s1=kc_validate(kc_val($attrib,'border-size'),2)?$attrib['border-size']:1;//宽度
			$color=$this->hex2rgb($im,$c1);//转换颜色
			
			switch(strtolower($border)){
				case 'rectangle':
					for($i=0;$i<$s1;$i++){//边框要一层层加
						imagerectangle($im,$i+$m1,$i+$m1,$w1-$i-$m1,$h1-$i-$m1,$color);
					}
				break;
				
				case 'fillet'://圆角矩形
					$d1=$m1+$r1;//设置距离
					for($i=0;$i<$s1;$i++){//边框要一层层加
						$d=$r1*2-$i*2;//直径
						//画线
						imageline($im,$d1,    $m1+$i,    $w1-$d1,$m1+$i,    $color);//上
						imageline($im,$w1-$m1-$i,$d1,    $w1-$m1-$i,$h1-$d1,$color);//右
						imageline($im,$d1,    $h1-$m1-$i,$w1-$d1,$h1-$m1-$i,$color);//下
						imageline($im,$m1+$i,    $d1,    $m1+$i,    $h1-$d1,$color);//左
						//画ARC
						imagearc($im,$d1,    $d1,    $d,$d,180,270,$color);//左上
						imagearc($im,$w1-$d1,$d1,    $d,$d,270,360,$color);//右上
						imagearc($im,$w1-$d1,$h1-$d1,$d,$d,  0, 90,$color);//右下
						imagearc($im,$d1,    $h1-$d1,$d,$d, 90,180,$color);//左下
					}
				break;

				case 'chamfer'://斜切矩形
					$d1=$m1+$r1-1;//设置距离
					for($i=0;$i<$s1;$i++){//边框要一层层加
						$d=$r1*2-$i*2;//直径
						//画线
						imageline($im,$d1,    $m1+$i,    $w1-$d1,$m1+$i,    $color);//上
						imageline($im,$w1-$m1-$i,$d1,    $w1-$m1-$i,$h1-$d1,$color);//右
						imageline($im,$d1,    $h1-$m1-$i,$w1-$d1,$h1-$m1-$i,$color);//下
						imageline($im,$m1+$i,    $d1,    $m1+$i,    $h1-$d1,$color);//左
					}
					$s2=ceil(sqrt(2)*$s1);
					for($i=0;$i<$s2;$i++){//边框要一层层加
						imageline($im,$w1-$d1-$i,$m1,$w1-$m1,$d1+$i,$color);//右上
						imageline($im,$w1-$m1,$h1-$d1-$i,$w1-$d1-$i,$h1-$m1,$color);//右下
						imageline($im,$d1+$i,    $h1-$m1,$m1,    $h1-$d1-$i,$color);//左下
						imageline($im,$m1,    $d1+$i,$d1+$i,    $m1,$color);//左上
					}
				break;
			}
		}
		//水印
		$watermark=kc_val($attrib,'watermark');
		if(isset($watermark{0})){
			//先判断文件是否存在
			if(is_file(ROOT.$watermark)){
				list($_width_water,$_height_water)=getimagesize(ROOT.$watermark);

				if(kc_validate($_width_water,2)&&kc_validate($_height_water,2)){
					$wx=kc_validate(kc_val($attrib,'watermark-x'),2)?$attrib['watermark-x']:10;
					$wy=kc_validate(kc_val($attrib,'watermark-y'),2)?$attrib['watermark-y']:10;
					$pct=kc_validate(kc_val($attrib,'watermark-opacity'),2)?$attrib['watermark-opacity']:100;
					$water_color=kc_val($attrib,'watermark-color') ? $attrib['watermark-color']:'FFFFFF';//透明色
					$wmsize=@getimagesize(ROOT.$watermark);
					$ext=strtolower(substr(strrchr(kc_val($wmsize,'mime'), '/'),1));
					if(in_array($ext,array('jpg','jpeg','gif','png'))){
						$fext_water= ($ext=='jpg') ? 'jpeg' : strtolower($ext);
					}else{
						return $s;
					}
					$func_water='imagecreatefrom'.$fext_water;
					$water=$func_water(ROOT.$watermark);//装载水印图片
					$color=$this->hex2rgb($water,$water_color);//转换颜色
					imagecolortransparent($water,$color);
					imagecopymerge($im,$water,$wx,$wy,0,0,$_width_water,$_height_water,$pct); // 左上
				}
			}
		}
		//插入字体
		$text=kc_val($attrib,'text');
		if(isset($text{0})){
			$tx=kc_validate(kc_val($attrib,'text-x'),2)?$attrib['text-x']:10;
			$ty=kc_validate(kc_val($attrib,'text-y'),2)?$attrib['text-y']:20;
			$s1=kc_validate(kc_val($attrib,'text-size'),2)?$attrib['text-size']:12;
			$a1=kc_validate(kc_val($attrib,'text-angle'),2)?$attrib['text-angle']:0;
			$f1=is_file(ROOT.kc_val($attrib,'text-font'))?$attrib['text-font']:'include/font/fzydzhjw.ttf';
			$c1=kc_validate(kc_val($attrib,'text-color'),13)?$attrib['text-color']:'000000';//颜色
			$color=$this->hex2rgb($im,$c1);//转换颜色
			imagettftext($im,$s1,$a1, $tx,$ty,$color,ROOT.$f1,$text);
		}
		//开启抗锯齿
		imageantialias($im,True);
                $file=new file;
                $file->md($newdir);//创建目录
		$func='image'.$fext;
		if($func($im,ROOT.$newimg)){//保存缩略图
			$s=$newimg;
		}
		return PATH.$s;
	}
	/**
	 * 文本内容生成为图片路径后返回图片地址
	 * @param string $s 内容字符串
	 * @param array $attrib 文本属性
	 * @param bool $is 文件存在时是否覆盖已有图 true覆盖 false 不覆盖
	 * @return string 文本转图片后的图片路径
	 */
	public function str2image($s,$attrib,$is=false){
		$s1=kc_validate(kc_val($attrib,'text-size'),2)?$attrib['text-size']:20;
		$w1=kc_validate(kc_val($attrib,'text-width'),2)?$attrib['text-width']:0;//输出宽度
		$h1=kc_validate(kc_val($attrib,'text-height'),2)?$attrib['text-height']:0;//输出高度
		$f1=is_file(ROOT.kc_val($attrib,'text-font'))?$attrib['text-font']:'include/font/fzydzhjw.ttf';
		$c1=kc_validate(kc_val($attrib,'text-color'),13)?$attrib['text-color']:'000000';//颜色
		$b1=kc_validate(kc_val($attrib,'text-bgcolor'),13)?substr($attrib['text-bgcolor'],1):'FFFFFF';//背景色
		//文件路径
		$md5path=md5($s.','.$s1.','.$w1.','.$h1.','.$f1.','.$c1.','.$b1);
		$filepath=PATH_UP.'/'.PATH_UP_IMAGE.'/text/'.substr($md5path,0,1).'/'.substr($md5path,1,1).'/'.$md5path.'.png';
		//如果已经生成过这个文件的话，直接返回地址。
		if (is_file(ROOT.$filepath) && !$is) {
			return PATH.$filepath;
		}
		$len=strlen($s);//中文算3个，这样不仅省事，而且有余地
		$width=ceil($s1*$len*2);
		$height=ceil($s1*2.2);
		//创建空白文档
		$im=imagecreatetruecolor($width,$height);
		//填充背景颜色
		$bg=$this->hex2rgb($im,$b1);
		imagefill($im,0,0,$bg);
		//写文本到图片
		$color=$this->hex2rgb($im,$c1);//转换颜色
		imagettftext($im,$s1,0,0,$s1+2,$color,ROOT.$f1,$s);
		//剪切字体到新的文档里
		$x1=$width;$y1=$height;//最小值
		$x2=0;$y2=0;//最大值
		//RGB颜色
		$rr=hexdec(substr($b1,0,2));
		$gg=hexdec(substr($b1,2,2));
		$bb=hexdec(substr($b1,4,2));
		$er=5;//误差值
		$rr1=$rr-$er;$rr2=$rr+$er;
		$gg1=$gg-$er;$gg2=$gg+$er;
		$bb1=$bb-$er;$bb2=$bb+$er;
		for($w=0;$w<$width;$w++){
			for ($h=0;$h<$height;$h++){
				//验证图片
				$rgb = imagecolorat($im,$w,$h);
				$r = ($rgb >> 16) & 0xFF;
				$g = ($rgb >> 8) & 0xFF;
				$b = $rgb & 0xFF;
				if (($r>$rr2 || $r<$rr1) || ($g>$gg2 || $g<$gg1) || ($b>$bb2 || $b<$bb1)) {
					$x1=$x1>=$w ? $w : $x1;
					$y1=$y1>=$h ? $h : $y1;
					$x2=$x2<=$w ? $w : $x2;
					$y2=$y2<=$h ? $h : $y2;
				}
			}
		}
		//新建空白图
		$w2=empty($w1) ? $x2-$x1+1 : $w1;
		$h2=empty($h1) ? $y2-$y1+1 : $h1;
		$new=imagecreatetruecolor($w2,$h2);
		//填充背景颜色
		imagefill($new,0,0,$bg);
		//拷贝文本到图库
		imagecopyresampled($new,$im,0,0,$x1,$y1,$w2,$h2,$x2-$x1+1,$y2-$y1+1);
		//创建目录
		 $file=new file;
         $file->md(dirname($filepath));
		//保存图像
		imagepng($new,ROOT.$filepath);
		return PATH.$filepath;
	}
	
	public function hex2rgb($im,$color,$pct=100){
		$color= $color{0}=='#'?substr($color,1):$color;
		if(strlen($color)!=6){
			return imagecolorallocate($im,0,0,0);
		}else{
			$color=strtoupper($color);
		}

		$array=array();
		for ($i=0;$i<3;$i++){
			$n=hexdec(substr($color,(2*$i),2));
			$array[$i]=round($n +(255-$n)*(1-$pct/100));
		}
		return imagecolorallocate($im,$array[0],$array[1],$array[2]);
	}
}

?>